// Registry.cpp: implementation of the CRegistry class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Registry.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

/////////////////////////////////////////////////////////
//////// CRegistryBase functions 
CRegistryBase::CRegistryBase()
{
	bAutoFlush = FALSE;
}
CRegistryBase::~CRegistryBase()
{
}
/////////////////////////////////////////////////////////
//////// general paths functions 
void CRegistryBase::Trim(CString *strRegPath,TCHAR ch)
{
	strRegPath->TrimLeft(ch);
	strRegPath->TrimRight(ch);
}
BOOL CRegistryBase::AppendPath(CString *strRegTarget,LPCTSTR lpszRegSrc)
{
    Trim(strRegTarget);
    CString src = lpszRegSrc;
	Trim(&src);
	*strRegTarget += _T('\\');
	*strRegTarget += src;
	Trim(strRegTarget);
	return TRUE;
}
/////////////////

////////////////////////////////////////////////////////////////////
///////////////// (look in ".h")
BOOL CRegistryBase::FormulatePath(CString *strRegPath,HKEY *hKey)
{
	CString str = *strRegPath;
	Trim(&str);
	strRegPath->Empty();

	int place = str.Find(_T('\\'));
	if (place==-1) 
		place=str.GetLength();//only includes a root item
	CString strRoot = str.Left(place);
	if (!StringRoot2key( strRoot, hKey))	{
		CMessage::Error("contains no such root");
		return FALSE;
	}
	*strRegPath = str.Mid(place);
	Trim(strRegPath);
	return TRUE;
}


////////////////////////////////////////////////////////
///////// Enumeration functions
BOOL CRegistryBase::Enum( HKEY hRootKey,CString *strRegPath,CStringArray *strKeyNames,CStringArray *strClassNames,BOOL bFullPath)
{
	HKEY hNewKey;
	if (!Open( hRootKey, strRegPath,&hNewKey))   
		return FALSE;

	TCHAR strKeyName[MAX_PATH*2],strClassName[MAX_PATH*2];
	DWORD dwNameSize=sizeof(strKeyName),dwClassSize=sizeof(strClassName);
    DWORD i=0;
	
	for (LONG result = ERROR_SUCCESS ; 
		((result==ERROR_SUCCESS) && (result!=ERROR_NO_MORE_ITEMS)) ;
		  i++)
	{
		dwNameSize=sizeof(strKeyName);
		dwClassSize=sizeof(strClassName);

        result = ::RegEnumKeyEx(hNewKey, i,strKeyName, &dwNameSize,NULL,strClassName, &dwClassSize,NULL);
        if (result == (DWORD)ERROR_SUCCESS) //suceeded getting name
        {
			strKeyNames->Add((LPCTSTR)strKeyName);
			if (strClassNames)  strClassNames->Add((LPCTSTR)strClassName);
		}
	}//end "for"

	Close(hNewKey);
	return TRUE;
}
BOOL CRegistryBase::Enum(CString *strRegPath,CStringArray *strKeyNames,CStringArray *strClassNames,BOOL bFullPath)
{
	CString str=*strRegPath;
	HKEY hRootKey;
	if (!FormulatePath(&str,&hRootKey))  return 0;
	return Enum(hRootKey,&str,strKeyNames,strClassNames,bFullPath);
}
BOOL CRegistryBase::Enum1Level(CString *strRegPath,CStringArray *strKeyNames,BOOL bFullPath)
{
	CWaitCursor  wait;
	CString strBackUp=*strRegPath,strPath2Use=strBackUp;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&strPath2Use,&hRootKey))  
		return FALSE;

	if (!Open(hRootKey,&strPath2Use,&hNewKey))   
		return FALSE;

	TCHAR strKeyName[MAX_PATH*2];
	DWORD dwNameSize;
    DWORD i=0;

	for (LONG result = ERROR_SUCCESS ; 
		((result==ERROR_SUCCESS) && (result!=ERROR_NO_MORE_ITEMS)) ;
		  i++)
	{
		dwNameSize=sizeof(strKeyName);
		result = ::RegEnumKeyEx(hNewKey, i,strKeyName, &dwNameSize,NULL,NULL ,NULL ,NULL);
        if (result == (DWORD)ERROR_SUCCESS) //suceeded getting name
        {
			if (!bFullPath) //dont add full path
				strKeyNames->Add((LPCTSTR)strKeyName);
			else
			{
				CString tmp=strBackUp;
				AppendPath(&tmp,(LPCTSTR)strKeyName);
				strKeyNames->Add(tmp);
			}
		}
	}//end "for"

	Close(hNewKey);
	return TRUE;
}

////////////////////////////////////////////////////////
///////// Open Keys functions
///////////////////
BOOL CRegistryBase::Open(CString *strPath,HKEY *pNewKey,REGSAM samDesired)
{
	HKEY hRootKey;
	if (!FormulatePath(strPath,&hRootKey)) return 0;
	return Open(hRootKey,strPath,pNewKey,samDesired);	
}
BOOL CRegistryBase::Open(HKEY hKey,CString *strPath,HKEY *pNewKey,REGSAM samDesired)
{
	*pNewKey=NULL;
	LONG lResult=ERROR_SUCCESS;
	
	if (strPath->GetLength())//not in root key
		  lResult = ::RegOpenKeyEx(hKey,(LPCTSTR)*strPath,0,samDesired,pNewKey);
	else  *pNewKey=hKey;//we are in root key dont need to open
	return (lResult==ERROR_SUCCESS);
}
BOOL CRegistryBase::Close(HKEY hKey)
{
	return Close(&hKey);
}
BOOL CRegistryBase::Close(HKEY *hKey)
{
	LONG lResult=ERROR_SUCCESS;
	if (bAutoFlush) Flush(*hKey);//flushing this key

	if (!IsRoot(hKey)) //closing only is this is not the root
		lResult = RegCloseKey(*hKey);
	return (lResult==ERROR_SUCCESS);
}

////////////////////////////////////////////////////////
///////// Flush function : immediate write
BOOL CRegistryBase::Flush(HKEY hKey)
{
	LONG lResult = RegFlushKey(hKey);
	return (lResult==ERROR_SUCCESS);
}
////////////////////////////////////////////////////////

////////////////////////////////////////////////////////
///////// Conversion between strings that represent root-keys
//to root keys and vice-versa. (protected functions)
CString CRegistryBase::Rootkey2String(HKEY *hKey)
{
	CString strResult=_T("");
	Rootkey2String(&strResult,hKey);
	return strResult;
}
BOOL CRegistryBase::Rootkey2String(CString *strRegSrc,HKEY *hKey)
{
	if (*hKey == HKEY_CLASSES_ROOT)
	{
		*strRegSrc=_T("HKEY_CLASSES_ROOT");
		return TRUE;
	}	
	else if (*hKey == HKEY_CURRENT_CONFIG)
	{
		*strRegSrc=_T("HKEY_CURRENT_CONFIG");
		return TRUE;
	}	
	else if (*hKey == HKEY_CURRENT_USER)
	{
		*strRegSrc=_T("HKEY_CURRENT_USER");
		return TRUE;
	}
	else if (*hKey == HKEY_LOCAL_MACHINE)
	{
		*strRegSrc=_T("HKEY_LOCAL_MACHINE");
		return TRUE;
	}
	else if (*hKey == HKEY_USERS)
	{
		*strRegSrc=_T("HKEY_USERS");
		return TRUE;
	}
	else if (*hKey == HKEY_DYN_DATA)
	{
		*strRegSrc=_T("HKEY_DYN_DATA");
		return TRUE;
	}
	else if (*hKey == HKEY_PERFORMANCE_DATA)
	{
		*strRegSrc=_T("HKEY_PERFORMANCE_DATA");
		return TRUE;
	}
	return FALSE;
}
BOOL CRegistryBase::StringRoot2key(LPCTSTR lpszRegPath,HKEY *hKey)
{
	CString strPath = lpszRegPath , strRoot;
	Trim(&strPath);
	GetLevel( 0, lpszRegPath, &strRoot);
	if (strRoot.IsEmpty())	return FALSE;

	if (!strRoot.CompareNoCase(_T("HKEY_CLASSES_ROOT"))) 
	{
		*hKey= HKEY_CLASSES_ROOT;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_CURRENT_CONFIG")))
	{
		*hKey= HKEY_CURRENT_CONFIG;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_CURRENT_USER")))
	{
		*hKey=HKEY_CURRENT_USER;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_LOCAL_MACHINE")))
	{
		*hKey=HKEY_LOCAL_MACHINE;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_USERS")))
	{
		*hKey=HKEY_USERS;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_DYN_DATA")))
	{
		*hKey=HKEY_DYN_DATA;
		return TRUE;
	}
	else if (!strRoot.CompareNoCase(_T("HKEY_PERFORMANCE_DATA")))
	{
		*hKey=HKEY_PERFORMANCE_DATA;
		return TRUE;
	}
	return FALSE;
}

////////////////////////////////////////////////////////
///////  IsRoot functions
BOOL CRegistryBase::IsRoot(HKEY hKey,CString strRegPath)
{
	return IsRoot(&hKey,&strRegPath);
}
BOOL CRegistryBase::IsRoot(HKEY *hKey,CString *strRegPath)
{
	if (!((*hKey == HKEY_CLASSES_ROOT)  || 
		  (*hKey == HKEY_CURRENT_CONFIG) || 
		  (*hKey == HKEY_CURRENT_USER) ||
		  (*hKey == HKEY_LOCAL_MACHINE) || 
		  (*hKey == HKEY_PERFORMANCE_DATA) || 
		  (*hKey == HKEY_USERS) ||
		  (*hKey == HKEY_DYN_DATA)))   return FALSE;//not root

	if (!strRegPath) //pointer is NULL : this is root
		return TRUE;
	//string contains no characters
	if (!strRegPath->GetLength())  return TRUE;//this is root		
	return FALSE;
}
BOOL CRegistryBase::IsRoot(CString *strRegPath)
{
	return IsRoot(*strRegPath);
}
BOOL CRegistryBase::IsRoot(LPCTSTR lpszRegPath)
{
	CString str = lpszRegPath;
	Trim(&str);
	if (!str.CompareNoCase(_T("HKEY_CLASSES_ROOT")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_CURRENT_CONFIG")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_CURRENT_USER")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_LOCAL_MACHINE")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_USERS")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_DYN_DATA")))
		return TRUE;
	if (!str.CompareNoCase(_T("HKEY_PERFORMANCE_DATA")))
		return TRUE;
	return FALSE;
}
////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////
BOOL CRegistryBase::IsKeyHasSubKeys(LPCTSTR lpszRegPath)
{
	DWORD dwSubKeys = GetSubKeyCount(lpszRegPath);
	return (dwSubKeys>0);
}
BOOL CRegistryBase::IsKeyHasSubKeys(HKEY hRegKey)
{	
	DWORD dwSubKeys = GetSubKeyCount(hRegKey);
	return (dwSubKeys>0);
}
int CRegistryBase::GetSubKeyCount(LPCTSTR lpszRegPath)
{
	CString strPath = lpszRegPath;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&strPath,&hRootKey))  return -1;
	if (!Open(hRootKey,&strPath,&hNewKey))  return -1;
	
	int iCount = GetSubKeyCount(hNewKey);
	Close(&hNewKey);
	return iCount;
}
int CRegistryBase::GetSubKeyCount(HKEY hKey)
{
	DWORD dwSubKeys = 0;
	if (::RegQueryInfoKey(hKey,NULL,NULL,NULL,&dwSubKeys,
					NULL,NULL,NULL,NULL,NULL,NULL,NULL)==ERROR_SUCCESS)
			return (int)dwSubKeys;
	return -1;
}

///////////////////////////////////////////////////////////
/////// IsExistValue Functions
BOOL CRegistryBase::IsExistValue(LPCTSTR lpszKeyName,LPCTSTR lpszValue)
{
	if (!lpszKeyName)	return FALSE;
	CString strRegPath = lpszKeyName;
	Trim(&strRegPath);
	HKEY hNewKey;
	if (Open(&strRegPath,&hNewKey)) {
		BOOL bFound  = IsExistValue( &hNewKey,lpszValue);
		Close(&hNewKey);
		return bFound;
	}
	return FALSE;
}
BOOL CRegistryBase::IsExistValue(HKEY *hKey , LPCTSTR lpszValue)
{
	TCHAR lpValueName[MAX_PATH+1];

	int nIndex = 0;
	DWORD cbData;
	LONG result;
//CString s;

	do {
		cbData = MAX_PATH;
		::memset(this,0,sizeof(lpValueName));
		result = ::RegEnumValue(*hKey , nIndex , lpValueName,
					&cbData,NULL,NULL,NULL,NULL);
		if (result == ERROR_SUCCESS) {
//s.Format("index=%d name=%s",nIndex,lpValueName);
//msg(s);
	//		CString tmp = lpszValue;
	//		if (!tmp.CompareNoCase((LPCTSTR) lpValueName))
	//			return TRUE;
		}
		nIndex++;
	}
	while (result != ERROR_NO_MORE_ITEMS);

	return FALSE;//not found such item
}


BOOL CRegistryBase::IsExistValue(LPCTSTR lpszKeyName,CStringArray *arr, CWordArray *bArrFound)
{
	if (!lpszKeyName)	return FALSE;
	CString strRegPath = lpszKeyName;
	Trim(&strRegPath);
	HKEY hNewKey;
	if (Open(&strRegPath,&hNewKey)) {
		BOOL bFound = IsExistValue( &hNewKey , arr , bArrFound );
	//msg("before close");
		Close(&hNewKey);
	//	msg("after close");
		return bFound;
	}
	return FALSE;
}
BOOL CRegistryBase::IsExistValue(HKEY *hKey , CStringArray *arr , CWordArray *bArrFound)
{
	TCHAR lpValueName[MAX_PATH+1];
	int iTotal = arr->GetSize();
	bArrFound->RemoveAll();
	while (iTotal--) bArrFound->Add(0);

	int nIndex = 0;
	DWORD cbData;
	LONG result;
//CString s;

	do {
		cbData = MAX_PATH;
		::memset(this,0,sizeof(lpValueName));
		result = ::RegEnumValue(*hKey , nIndex , lpValueName,
					&cbData,NULL,NULL,NULL,NULL);
		if (result == ERROR_SUCCESS) {
//s.Format("index=%d name=%s",nIndex,lpValueName);
//msg(s);
			CString val = lpValueName;
			for (int i = 0, iCount=arr->GetSize() ; i<iCount ; i++ ) {
				CString tmp = arr->GetAt(i);
				if (!tmp.CompareNoCase(val))
					bArrFound->SetAt(i,1);
			}
		}
		nIndex++;
	}
	while (result != ERROR_NO_MORE_ITEMS);

	return FALSE;//not found such item
}


///////////////////////////////////////////////////////////
////////////// Level functions
int CRegistryBase::CountLevels( LPCTSTR lpszRegPath)
{
	CString sPath = lpszRegPath;
	Trim(&sPath);
	int iTotal = sPath.GetLength();
	if (!iTotal)  return 0;

	for (int iCount=1 , i=0 ; i<iTotal ; i++)
		if ( sPath.GetAt(i) == _T('\\'))  iCount++;
	return iCount;
}
BOOL CRegistryBase::GetLevel(int iLevel, LPCTSTR lpszRegPath, CString *strRegOut)
{
	CString sPath = lpszRegPath;
	Trim(&sPath);
	int iCount = CountLevels( (LPCTSTR) sPath );
	if (!iCount)  return 0;//no levels in name
	if ((iLevel<0) || (iLevel>=iCount))  return FALSE;//no such level
	//out of boundary

	for ( int i=0 , tmp=0 ; tmp < iLevel ; i++)
		if (sPath[i] == _T('\\'))  tmp++;

	sPath = sPath.Mid(i);
	i = sPath.Find(_T('\\'));
	if (i>=0)	sPath = sPath.Left(i);
	
	*strRegOut = sPath;
	return TRUE;
}
BOOL CRegistryBase::GetLastLevel( LPCTSTR lpszRegPath , CString *strRegOut)
{
	CString path = lpszRegPath;
	Trim(&path);
	int iLevels = CountLevels( (LPCTSTR) path);
	if (!iLevels)  return FALSE;
	return GetLevel( iLevels-1, path, strRegOut);
}
BOOL CRegistryBase::LevelInPath( LPCTSTR lpszRegPath , CString *strWantedKey)
{
	CString path = lpszRegPath;
	Trim(&path);
	int iLevels = CountLevels( (LPCTSTR) path);
	if (!iLevels)  return FALSE;
	
	CString str;
	for (int i=0 ; i<iLevels ; i++)
		if ((GetLevel( i, (LPCTSTR) path, &str)) && (!strWantedKey->CompareNoCase(str)))
			return TRUE;
	return FALSE;
}
BOOL CRegistryBase::GetLevelBeforeLevel( LPCTSTR lpszRegPath , LPCTSTR lpszBeforeLevel, CString *strOut, BOOL bGiveRoot)
{
	CString path = lpszRegPath;
	Trim(&path);
	int iLevels = CountLevels( (LPCTSTR) path);
	if (iLevels<2)  return FALSE;//1 level only so theres non before
	
	CString str;
	for (int i=1 ; i<iLevels ; i++)
		if ((GetLevel( i, (LPCTSTR) path, &str)) && (!str.CompareNoCase(lpszBeforeLevel))) {
			if ((((i==1) && (bGiveRoot)) || (i>1)) && (GetLevel( i-1, (LPCTSTR) path, &str))) {
				*strOut=str;
				return TRUE;
			}
		}
	return FALSE;
}
BOOL CRegistryBase::GetLevelAfterLevel( LPCTSTR lpszRegPath , LPCTSTR lpszAfterLevel, CString *strOut)
{
	CString path = lpszRegPath ;
	Trim(&path);
	int iLevels = CountLevels(path);
	if (iLevels<2)  return FALSE;//no levels after level

	CString str;
	for (int i=0 ; i<iLevels-1 ; i++)
		if ((GetLevel( i,(LPCTSTR) path, &str)) && (!str.CompareNoCase(lpszAfterLevel))) {
			if (GetLevel( i+1, (LPCTSTR) path, &str)) {
				*strOut=str;
				return TRUE;
			}
		}
	return FALSE;
}
BOOL CRegistryBase::RemoveLastItem(CString *strRegPath)
{
	CString str = *strRegPath;
	Trim(&str);
	int place=str.ReverseFind(_T('\\'));
	if (place==-1)	*strRegPath = str;
	else	*strRegPath=str.Left(place);
	Trim(strRegPath);
	return TRUE;
}
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////
// CRegistry Construction/Destruction
//////////////////////////////////////////////////////////////////////
CRegistry::CRegistry()
{
}
CRegistry::~CRegistry()
{
}
/////////////////////////////////////////////////////////////
////////// IsExistKey functions
//checks whether a key exists in registry ...
BOOL CRegistry::IsExistKey(LPCTSTR lpszRegPath)
{
	CString path = lpszRegPath;
	HKEY hNewKey;
	if (Open(&path,&hNewKey)){
		Close(&hNewKey);
		return TRUE;
	}
	return FALSE;
}
//finds a key in some level returns the (level+1)
BOOL CRegistry::IsExistKey(const CString &strWantedKey,const CString &strRegPath,int iLevelRequired)
{
	CString path=strRegPath,key=strWantedKey;
	return IsExistKey(&path,&key,iLevelRequired);
}
BOOL CRegistry::IsExistKey(CString *strWantedKey,CString *strRegPath,int iLevelRequired)
{
	CString path=*strRegPath,sOut;
	int iTotalLevels = CountLevels( path);

	if (iLevelRequired==-1)
		for (int iLevel=0; iLevel<iTotalLevels ; iLevel++ )
		{
			if ((GetLevel( iLevel, (LPCTSTR) path, &sOut)) && 
				(!strWantedKey->CompareNoCase(sOut)))
					return  iLevel+1;
		}
	else if (iLevelRequired<iTotalLevels)
	{
		if ((GetLevel( iLevelRequired, (LPCTSTR) path, &sOut)) && 
				(!strWantedKey->CompareNoCase(sOut)))
				return iLevelRequired+1;
	}
	return FALSE;
}

//takes CStringArray

//return index+1 of the item where first key was found
BOOL CRegistry::IsExistKey(const CString &strWantedKey,CStringArray *strKeyNames,int iLevelRequired)
{
	CString strWanted=strWantedKey;
	return IsExistKey(&strWanted,strKeyNames);
}
BOOL CRegistry::IsExistKey(CString *strWantedKey,CStringArray *strKeyNames,int iLevelRequired)
{
	CString strWanted=*strWantedKey,path;
	for (int iCount=strKeyNames->GetSize(),i=iCount-1 ; i>=0 ;i--)
	{
		path=strKeyNames->GetAt(i);
		if (IsExistKey(&strWanted,&path,iLevelRequired))
			return TRUE;
	}
	return FALSE;
}
///////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
/////////// Functions to Delete Keys and Values ....

//if on windows NT : (unicode systems) indlude atl for recursive
//deletion of registry keys on with "CRegKey"
//(no need to reinvent the wheel)
#ifdef UNICODE
	#include <atlbase.h>
#endif

BOOL CRegistry::DeleteKey( LPCTSTR lpszRegPath ) {
	CString path = lpszRegPath ;
	HKEY hRootKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

#ifdef UNICODE
	CRegKey atlRegKey;
	atlRegKey.Attach(hRootKey);
	return (atlRegKey.RecurseDeleteKey((LPCTSTR)path)==ERROR_SUCCESS);
#else
	return DeleteKey(&hRootKey, path);
#endif

}
BOOL CRegistry::DeleteKey( HKEY *hKey, LPCTSTR lpszRegPath) 
{
	if (::RegDeleteKey( *hKey, lpszRegPath ) == ERROR_SUCCESS)
		return TRUE;
	return FALSE;
}

BOOL CRegistry::DeleteValue( LPCTSTR lpszRegPath, LPCTSTR lpszValueName) 
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;
	
	if (!Open(hRootKey,&path,&hNewKey,KEY_SET_VALUE ))
		return FALSE;
	
	BOOL bResult = DeleteValue( &hNewKey, lpszValueName);

	Close(&hNewKey);
	return bResult;
}
BOOL CRegistry::DeleteValue(HKEY *hKey, LPCTSTR lpszValueName) {
	if (::RegDeleteValue( *hKey, lpszValueName )== ERROR_SUCCESS)
		return TRUE;
	return FALSE;
}
//////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
/////////// Functions to create keys
BOOL CRegistry::CreateKey( LPCTSTR lpszRegPath, LPCTSTR lpszRegNewKey)
{
	CString path = lpszRegPath ;
	AppendPath( &path , lpszRegNewKey);
	return CreateKey(path);
}
BOOL CRegistry::CreateKey( LPCTSTR lpszRegPath)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath( &path, &hRootKey))  return FALSE;

	BOOL bResult = CreateKey( &hRootKey, path);
	Close( &hNewKey);
	return bResult;
}
BOOL CRegistry::CreateKey( HKEY *hKey, LPCTSTR lpszRegPath)
{
	HKEY hKeyResult;
	if (!lpszRegPath)  return FALSE;
	REGSAM regSam = KEY_CREATE_SUB_KEY|KEY_ENUMERATE_SUB_KEYS;
	LONG result = ::RegCreateKeyEx( *hKey, lpszRegPath , 0 ,
							NULL , REG_OPTION_NON_VOLATILE , regSam ,
							NULL , &hKeyResult ,
							NULL);
	Close(&hKeyResult);
	return (result == ERROR_SUCCESS);
}
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
//////////////  Functions to set values
BOOL CRegistry::SetValueInt( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,int iData)
{
	size_t size = sizeof(int);
	return SetValue(lpszRegPath,lpszValueName,(LPVOID)&iData,size);
}
BOOL CRegistry::SetValueFloat( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,float fData)
{
	size_t size = sizeof(float);
	return SetValue(lpszRegPath,lpszValueName,(LPVOID)&fData,size);
}
BOOL CRegistry::SetValueString( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,LPCTSTR lpszData)
{
	CString strData = lpszData;
	size_t size = strData.GetLength() * sizeof(TCHAR);
	return SetValue(lpszRegPath,lpszValueName,(LPVOID)lpszData,size,REG_SZ);
}
BOOL CRegistry::SetValueUint( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,DWORD dwData)
{
	size_t size = sizeof(UINT);
	return SetValue(lpszRegPath,lpszValueName,(LPVOID)&dwData,size,REG_DWORD);
}

BOOL CRegistry::SetValue( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,LPVOID pData ,size_t size ,DWORD dwType)
{
	if (size <= 0)  return FALSE; //invalid value name
	CString strRegPath = lpszRegPath;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&strRegPath,&hRootKey))
		return FALSE;//invalide root key
	if (Open(hRootKey,&strRegPath,&hNewKey,KEY_SET_VALUE)) {
		BOOL bDone = 
			SetValue( &hNewKey, lpszValueName, pData , size, dwType);
		Close(&hNewKey);
		return bDone;
	}
	return FALSE;
}

BOOL CRegistry::SetValue(HKEY *hKey,LPCTSTR lpszValueName,LPVOID pData ,size_t size,DWORD dwType )
{
	LONG lResult = ::RegSetValueEx( *hKey, lpszValueName, NULL, dwType, (const BYTE*) pData, size);
	return (lResult==ERROR_SUCCESS);
}
/// End functions to set value
///////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////
//////// Create Value functions ...
BOOL CRegistry::CreateValueString( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,CString *strData)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

	if (!Open(hRootKey,&path,&hNewKey,KEY_WRITE|KEY_READ))
		return FALSE;

	BOOL bResult = CreateValueString(&hNewKey,lpszValueName,strData);
	Close(&hNewKey);
	return bResult;
}
BOOL CRegistry::CreateValueDword( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,DWORD dwData)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

	if (!Open(hRootKey,&path,&hNewKey,KEY_WRITE|KEY_SET_VALUE ))
		return FALSE;

	BOOL bResult = CreateValueDword( &hNewKey, lpszValueName, dwData);
	Close(&hNewKey);
	return bResult;
}
BOOL CRegistry::CreateValueInt( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,int iData)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

	if (!Open(hRootKey,&path,&hNewKey,KEY_WRITE|KEY_READ))
		return FALSE;

	BOOL bResult = CreateValueInt( &hNewKey, lpszValueName, iData);
	Close(&hNewKey);
	return bResult;
}
BOOL CRegistry::CreateValueFloat( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,float fData)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

	if (!Open(hRootKey,&path,&hNewKey,KEY_WRITE|KEY_READ))
		return FALSE;

	BOOL bResult = CreateValueFloat( &hNewKey, lpszValueName, fData);
	Close(&hNewKey);
	return bResult;
}
BOOL CRegistry::CreateValueString(HKEY *hKey,LPCTSTR lpszValueName,CString *strData)
{
	DWORD dwType = REG_SZ;
	DWORD cbSize = (strData->GetLength()+1);//*sizeof(TCHAR)
	return CreateValue( hKey, lpszValueName,(LPBYTE)(LPCTSTR) *strData, cbSize, REG_SZ);
}
BOOL CRegistry::CreateValueInt(HKEY *hKey,LPCTSTR lpszValueName,int iData)
{
	return CreateValue( hKey, lpszValueName, (LPBYTE)&iData, sizeof(int), REG_BINARY);
}
BOOL CRegistry::CreateValueDword(HKEY *hKey,LPCTSTR lpszValueName,DWORD dwData)
{
	return CreateValue( hKey, lpszValueName, (LPBYTE)&dwData, sizeof(DWORD), REG_DWORD);
}
BOOL CRegistry::CreateValueFloat(HKEY *hKey,LPCTSTR lpszValueName,float fData)
{
	return CreateValue( hKey, lpszValueName,(LPBYTE)&fData, sizeof(float), REG_BINARY);
}
BOOL CRegistry::CreateValue(HKEY *hKey,LPCTSTR lpszValueName,
				LPBYTE pData,DWORD cbSize,DWORD dwType)
{
	LONG result = ::RegSetValueEx( *hKey, lpszValueName, 0,
					 dwType, pData, cbSize);
	return (result==ERROR_SUCCESS);
}

BOOL CRegistry::CreateValue( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,
				LPBYTE pData,DWORD cbSize,DWORD dwType)
{
	CString path = lpszRegPath ;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&path,&hRootKey))  return FALSE;

	if (!Open(hRootKey,&path,&hNewKey,KEY_WRITE|KEY_READ))
		return FALSE;

	BOOL bResult = CreateValue( &hNewKey, lpszValueName, pData, cbSize,dwType);
	Close(&hNewKey);
	return bResult;
}
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////
////////// GetValue  functions
BOOL CRegistry::GetValueInt( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,int *iResult)
{
	LPBYTE p=NULL;
	DWORD dwLength=0;
	DWORD dwType;
	if (GetValue( lpszRegPath, lpszValueName, &p, &dwType, &dwLength))
	{
		switch  (dwType)
		{
		case REG_BINARY:
		case REG_DWORD:
			::CopyMemory( iResult , p , min( dwLength,sizeof(int)) );
			break;
		}
		delete [] p;//deletes the buffer
		return TRUE;
	}

	return FALSE;
}
BOOL CRegistry::GetValueUInt( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,UINT *uResult)
{
	LPBYTE p = NULL;
	DWORD dwLength=0;
	DWORD dwType;
	if (GetValue(  lpszRegPath, lpszValueName, &p, &dwType, &dwLength))
	{
		switch  (dwType)
		{
		case REG_BINARY:
		case REG_DWORD:
			::CopyMemory( uResult , p , min( dwLength,sizeof(UINT)) );
			break;
		}
		delete [] p;//deletes the buffer
		return TRUE;
	}
	return FALSE;
}
BOOL CRegistry::GetValueFloat( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,float *fResult)
{
	LPBYTE p = NULL;
	DWORD dwLength=0;
	DWORD dwType;
	if (GetValue( lpszRegPath, lpszValueName, &p, &dwType, &dwLength))
	{
		switch  (dwType)
		{
		case REG_BINARY:
			::CopyMemory( fResult , p , min( dwLength,sizeof(float)) );
			break;
		}
		delete [] p;//deletes the buffer
		return TRUE;
	}
	return FALSE;
}
BOOL CRegistry::GetValueDefault(LPCTSTR lpszRegPath,CString *strResult)
{
	return GetValueString( lpszRegPath, _T("") , strResult);
}
BOOL CRegistry::GetValueString( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,CString *strResult)
{
	LPBYTE p = NULL;
	DWORD dwLength=0;
	DWORD dwType;
	if (GetValue( lpszRegPath, lpszValueName, &p, &dwType, &dwLength))
	{
		switch  (dwType)
		{
		case REG_SZ:
		case REG_EXPAND_SZ:
			*strResult=(LPCTSTR) p;
			break;
		}
		delete [] p;//deletes the buffer
		return TRUE;
	}
	return FALSE;
}

BOOL CRegistry::GetValueSized( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,LPVOID pData,size_t size)
{
	DWORD dwType;
	DWORD dwDataGot = 0;
	LPBYTE p = NULL;
	if (GetValue( lpszRegPath, lpszValueName, &p, &dwType, &dwDataGot)) {
		if ((p) && (dwDataGot>0)) {
			::CopyMemory( pData,p, min(dwDataGot,size) );
		}
		if (p)  delete [] p;
		return TRUE;
	}
	return FALSE;
}

//gets a pointer 
//NOTICE !!!  The caller MUST DELETE the allocated MEMORY !!!
BOOL CRegistry::GetValue( LPCTSTR lpszRegPath, LPCTSTR lpszValueName,LPBYTE *pBuffer,DWORD *pdwType,DWORD *pdwDataGot)
{
	CString strRegPath = lpszRegPath;
	HKEY hRootKey,hNewKey;
	if (!FormulatePath(&strRegPath,&hRootKey)) return FALSE;
	if (!Open(hRootKey,&strRegPath,&hNewKey))  return FALSE;

	DWORD cbData = 0;
	// Get the length of the data 
	LONG  result = ::RegQueryValueEx( hNewKey, lpszValueName,
				NULL, pdwType, NULL , &cbData);
	
	if (result != ERROR_SUCCESS)
		return FALSE;
	
	LPBYTE p = new BYTE [ cbData+1];
	if (!p)  return FALSE;
	::ZeroMemory(p,cbData+1);
	
	//Get the data
	result = ::RegQueryValueEx( hNewKey, lpszValueName, NULL, 
			pdwType, p, &cbData);

	if (result == ERROR_SUCCESS)
	{
		*pBuffer = p;
		*pdwDataGot = cbData;
		Close(&hNewKey);	
		return TRUE;
	}

	delete [] p; //failed
	*pBuffer = NULL;
	Close(&hNewKey);	
	return FALSE;
}

///////////////////////////////////////////////////////////
