// WindowTip.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "WindowTip.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

void msg(CString err="",CString title="WindowTip")
{
	::MessageBox(::GetDesktopWindow(),err,title,MB_OK);
}

//////////////////////////////////////////////////
//////////   CNonClientMetrics
class CNonClientMetrics : public NONCLIENTMETRICS {
public:
	CNonClientMetrics() {
		cbSize = sizeof(NONCLIENTMETRICS);
		SystemParametersInfo(SPI_GETNONCLIENTMETRICS,0,this,0);
	}
};

/////////////////////////////////////////////////////////////////////////////
// CWindowTip
CWindowTip::CWindowTip()
{
	m_szMargins = CSize(4,2);
	// create font -- use system tooltip font
	CNonClientMetrics ncm;
	m_font.CreateFontIndirect(&ncm.lfStatusFont);

	m_ScreenSize = CSize (::GetSystemMetrics(SM_CXSCREEN),
						  ::GetSystemMetrics(SM_CYSCREEN));
	uTimerID = 0;
	iTimeOfTip = 2 ;//2 Seconds interval of showing tip
}

CWindowTip::~CWindowTip()
{
	StopTimer();
}

//static object to hold previous tip
CWindowTip* CWindowTip::m_pWndPrevTip = NULL;

BEGIN_MESSAGE_MAP(CWindowTip, CWnd)
	//{{AFX_MSG_MAP(CWindowTip)
	ON_WM_PAINT()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//////////////////
// Register class if needed
//
BOOL CWindowTip::PreCreateWindow(CREATESTRUCT& cs) 
{
	static CString sClassName;
//	if (sClassName.IsEmpty())
		sClassName = AfxRegisterWndClass(0);

	cs.lpszClass = sClassName;
	strClassName = sClassName;
	cs.style |= (WS_BORDER|WS_VISIBLE);//    // |WS_POPUP
	return CWnd::PreCreateWindow(cs);
}

BOOL CWindowTip::Create( CWnd* pParentWnd, UINT nID, BOOL bHidePrevTip)
{
	if (::IsWindow(m_hWnd))  return TRUE;
	this->bHidePrevTip = bHidePrevTip;

	BOOL bResult = CreateEx(0,//WS_EX_TOPMOST,// ,  // |WS_EX_TOOLWINDOW,//
			strClassName,NULL,
			WS_CHILD  ,// // ,//  |WS_POPUP
			CRect(CPoint(0,0),CSize(0,0)),
			CWnd::GetDesktopWindow(),nID);///pParentWnd
	return bResult;
}

BOOL CWindowTip::SetText(LPCTSTR lpszText,int xPos,int yPos)
{
	strText = lpszText;
	if (!strText.GetLength())
		return EndWindow();
	return StartWindow(xPos,yPos);
}

//////////////////
// CWindowTip is intended to be used on the stack,
// not heap, so don't auto-delete.
void CWindowTip::PostNcDestroy()
{
	// don't delete this
}

CPoint CWindowTip::GetProperWindowPos(const CSize m_RequestedSize)
{
	CPoint point(::GetMessagePos());


	return point;
}
CSize CWindowTip::GetWindowSize(LPCTSTR strText)
{
	CString text = strText ;
	CPaintDC dc(this);
	CSize size;
	
	int iCount=text.GetLength();
	::GetTextExtentPoint32(dc.m_hDC,(LPCTSTR) text, iCount, &size);
	
	for (int i=0,iNewLines=0 ; i<iCount ; i++)
		if (text.GetAt(i) == _T('\n')) iNewLines++;

	if (iNewLines) //there are several lines
	{
		size.cx /= iNewLines ;
		size.cy *= iNewLines ;
	}
	return size;
}

BOOL CWindowTip::StartWindow(int xPos,int yPos)
{
	ASSERT(::IsWindow(m_hWnd));//must already be created
	EndWindow();//end window in case is visible
	CreateTimer();

	CPaintDC dc(this);
	CRect rc;
	CString str = strText;
	CSize size = CSize(0,0);//GetProperWindowSize(strText);
	
	CFont* pOldFont = dc.SelectObject(&m_font);
	dc.DrawText(str, &rc, DT_CALCRECT|DT_CENTER|DT_VCENTER);
	dc.SelectObject(pOldFont);

	CPoint point(xPos,yPos);
	if ((xPos<0) || (yPos<0))
		point = GetProperWindowPos(size);

	rc.InflateRect(m_szMargins);
	SetWindowPos( NULL,point.x,point.y,rc.Width(),rc.Height(), SWP_SHOWWINDOW|SWP_SHOWWINDOW);
	MoveWindow(point.x,point.y,rc.Width(),rc.Height());

	ShowWindow(SW_SHOW);
	HidePrevWindow();
	SetLastWindow();
//	UpdateWindow();

	return TRUE;
}
BOOL CWindowTip::EndWindow()
{
	if (IsWindowVisible())
	{
		ShowWindow(SW_HIDE);
//		Invalidate();
//		UpdateWindow();
	}
	return StopTimer();
}
void CWindowTip::OnTimer(UINT nIDEvent) 
{
	if (uTimerID == nIDEvent)
	{
		if (iTimeOfTip == iElapedTime)
			EndWindow();
		iElapedTime++;
	}
	else CWnd::OnTimer(nIDEvent);
}

BOOL CWindowTip::CreateTimer()
{
	iElapedTime = 0;
	StopTimer();
	UINT nIDEvent=500;
	while (!uTimerID)
	{
		if (SetTimer(nIDEvent,1000,NULL))
			uTimerID = nIDEvent;
		nIDEvent++;
	}
	return TRUE;
}
BOOL CWindowTip::StopTimer()
{
	if (uTimerID)
	{
		KillTimer((int)uTimerID);
		uTimerID = 0;
	}
	return TRUE;
}
///////////////////////////////////////////////////////////

//////////////////
// Paint the text. Use system colors
void CWindowTip::OnPaint()
{
	CPaintDC dc(this);
	CRect rc;
	GetClientRect(&rc);
	CString str = strText;
	CBrush b(GetSysColor(COLOR_INFOBK)); // COLOR_WINDOW use tooltip bg color
	dc.FillRect(&rc, &b);

	// draw text
	dc.SetBkMode(TRANSPARENT);
	CFont* pOldFont = dc.SelectObject(&m_font);
	dc.SetTextColor(GetSysColor(COLOR_INFOTEXT)); // tooltip text color

	dc.DrawText(str, &rc, DT_CENTER|DT_VCENTER);
	dc.SelectObject(pOldFont);

//	BringWindowToTop();
//	SetWindowPos(&wndTopMost ,point.x,point.y,rc.Width(),rc.Height(), SWP_SHOWWINDOW|SWP_SHOWWINDOW);
}

void CWindowTip::JustifyText(CPaintDC *pDC, LPCTSTR pText , CRect *prc,int iAlign)
{
/*	int xStart , yStart = prc->top, iBreakCount;
	LPCTSTR pBegin,pEnd ;
	CSize size;

	do {
		iBreakCount = 0;
		while ( *pText == _T(' '))
			pText++; //skip blank spaces
	
		pBegin = pText;
		do {
			pEnd =pText;
			while ( (*pText != _T('\0')) && (*pText++ != _T(' ')) );
			if (*pText == _T('\0'))
				break;

			iBreakCount++;
			::SetTextJustification(pDC->m_hDC,0,0);
			::GetTextExtentPoint32(pDC->m_hDC,pBegin,pEnd-pBegin-1,&size);
		}
		while (size.cx<prc->right-prc->left); //while (size.cx<prc->Width());

		iBreakCount--;
		while (*(pEnd-1) == _T(' ')) 
		{
			pEnd--;
			iBreakCount--;
		}
		if ((*pText == _T('\0')) || (iBreakCount<=0))
			pEnd = pText;

		::SetTextJustification(pDC->m_hDC,0,0);
		::GetTextExtentPoint32(pDC->m_hDC,pBegin,pEnd-pBegin,&size);

		switch (iAlign)
		{
		case LEFT: xStart = prc->left;
			break;
		case RIGHT: xStart = prc->right -size.cx;
			break;
		case CENTER: xStart = (prc->right+prc->left-size.cx)/2;
			break;
		}	
		
		::TextOut(pDC->m_hDC,xStart,yStart,pBegin,pEnd-pBegin);
		yStart += size.cy;
		pText = pEnd;
	}// end do-while
	while ((*pText) && (yStart<prc->bottom));
*/
}

void CWindowTip::OnDestroy() 
{
	StopTimer();
	CWnd::OnDestroy();
}
void CWindowTip::OnLButtonDown(UINT nFlags, CPoint point) 
{
	EndWindow();
	
	CWnd::OnLButtonDown(nFlags, point);
}
